@@ -13,7 +13,7 @@
  * We are willing to use at most maintenance_work_mem memory space to keep
  * track of dead tuples.  We initially allocate an array of TIDs of that size,
  * with an upper limit that depends on table size (this limit ensures we don't
- * allocate a huge area uselessly for vacuuming small tables).  If the array
+ * allocate a huge area uselessly for vacuuming small tables).	If the array
  * threatens to overflow, we suspend the heap scan phase and perform a pass of
  * index cleanup and page compaction, then resume the heap scan with an empty
  * TID array.
@@ -24,12 +24,12 @@
  * the TID array, just enough to hold as many heap tuples as fit on one page.
  *
  *
- * Portions Copyright (c) 1996-2010, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2011, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $PostgreSQL: pgsql/src/backend/commands/vacuumlazy.c,v 1.136 2010/07/06 19:18:56 momjian Exp $
+ *	  src/backend/commands/vacuumlazy.c
  *
  *-------------------------------------------------------------------------
  */
@@ -46,7 +46,6 @@
 #include "commands/vacuum.h"
 #include "miscadmin.h"
 #include "pgstat.h"
-#include "portability/instr_time.h"
 #include "postmaster/autovacuum.h"
 #include "storage/bufmgr.h"
 #include "storage/freespace.h"
@@ -67,17 +66,6 @@
 #define REL_TRUNCATE_MINIMUM	1000
 #define REL_TRUNCATE_FRACTION	16
 
-/*
- * Timing parameters for truncate locking heuristics.
- *
- * These were not exposed as user tunable GUC values because it didn't seem
- * that the potential for improvement was great enough to merit the cost of
- * supporting them.
- */
-#define VACUUM_TRUNCATE_LOCK_CHECK_INTERVAL		20	/* ms */
-#define VACUUM_TRUNCATE_LOCK_WAIT_INTERVAL		50	/* ms */
-#define VACUUM_TRUNCATE_LOCK_TIMEOUT			5000		/* ms */
-
 /*
  * Guesstimation of number of dead tuples per page.  This is used to
  * provide an upper limit to memory allocated when vacuuming small
@@ -99,7 +87,7 @@ typedef struct LVRelStats
 	BlockNumber old_rel_pages;	/* previous value of pg_class.relpages */
 	BlockNumber rel_pages;		/* total number of pages */
 	BlockNumber scanned_pages;	/* number of pages we examined */
-	double		scanned_tuples;	/* counts only tuples on scanned pages */
+	double		scanned_tuples; /* counts only tuples on scanned pages */
 	double		old_rel_tuples; /* previous value of pg_class.reltuples */
 	double		new_rel_tuples; /* new estimated total # of tuples */
 	BlockNumber pages_removed;
@@ -112,7 +100,6 @@ typedef struct LVRelStats
 	ItemPointer dead_tuples;	/* array of ItemPointerData */
 	int			num_index_scans;
 	TransactionId latestRemovedXid;
-	bool		lock_waiter_detected;
 } LVRelStats;
 
 
@@ -166,8 +153,7 @@ lazy_vacuum_rel(Relation onerel, VacuumStmt *vacstmt,
 	BlockNumber possibly_freeable;
 	PGRUsage	ru0;
 	TimestampTz starttime = 0;
-	bool		scan_all;		/* should we scan all pages? */
-	bool		scanned_all;	/* did we actually scan all pages? */
+	bool		scan_all;
 	TransactionId freezeTableLimit;
 	BlockNumber new_rel_pages;
 	double		new_rel_tuples;
@@ -197,8 +183,6 @@ lazy_vacuum_rel(Relation onerel, VacuumStmt *vacstmt,
 	vacrelstats->old_rel_pages = onerel->rd_rel->relpages;
 	vacrelstats->old_rel_tuples = onerel->rd_rel->reltuples;
 	vacrelstats->num_index_scans = 0;
-	vacrelstats->pages_removed = 0;
-	vacrelstats->lock_waiter_detected = false;
 
 	/* Open all indexes of the relation */
 	vac_open_indexes(onerel, RowExclusiveLock, &nindexes, &Irel);
@@ -210,21 +194,6 @@ lazy_vacuum_rel(Relation onerel, VacuumStmt *vacstmt,
 	/* Done with indexes */
 	vac_close_indexes(nindexes, Irel, NoLock);
 
-	/*
-	 * Compute whether we actually scanned the whole relation. If we did, we
-	 * can adjust relfrozenxid.
-	 *
-	 * NB: We need to check this before truncating the relation, because that
-	 * will change ->rel_pages.
-	 */
-	if (vacrelstats->scanned_pages < vacrelstats->rel_pages)
-	{
-		Assert(!scan_all);
-		scanned_all = false;
-	}
-	else
-		scanned_all = true;
-
 	/*
 	 * Optionally truncate the relation.
 	 *
@@ -261,18 +230,19 @@ lazy_vacuum_rel(Relation onerel, VacuumStmt *vacstmt,
 		new_rel_tuples = vacrelstats->old_rel_tuples;
 	}
 
-	new_frozen_xid = scanned_all ? FreezeLimit : InvalidTransactionId;
+	new_frozen_xid = FreezeLimit;
+	if (vacrelstats->scanned_pages < vacrelstats->rel_pages)
+		new_frozen_xid = InvalidTransactionId;
 
 	vac_update_relstats(onerel,
 						new_rel_pages, new_rel_tuples,
 						vacrelstats->hasindex,
-						new_frozen_xid,
-						false);
+						new_frozen_xid);
 
 	/* report results to the stats collector, too */
 	pgstat_report_vacuum(RelationGetRelid(onerel),
-						  onerel->rd_rel->relisshared,
-						  new_rel_tuples);
+						 onerel->rd_rel->relisshared,
+						 new_rel_tuples);
 
 	/* and log the action if appropriate */
 	if (IsAutoVacuumWorkerProcess() && Log_autovacuum_min_duration >= 0)
@@ -317,10 +287,10 @@ static void
 vacuum_log_cleanup_info(Relation rel, LVRelStats *vacrelstats)
 {
 	/*
-	 * No need to log changes for temp tables, they do not contain data
-	 * visible on the standby server.
+	 * Skip this for relations for which no WAL is to be written, or if we're
+	 * not trying to support archive recovery.
 	 */
-	if (rel->rd_istemp || !XLogIsNeeded())
+	if (!RelationNeedsWAL(rel) || !XLogIsNeeded())
 		return;
 
 	/*
@@ -395,9 +365,9 @@ lazy_scan_heap(Relation onerel, LVRelStats *vacrelstats,
 	 * of pages.
 	 *
 	 * Before entering the main loop, establish the invariant that
-	 * next_not_all_visible_block is the next block number >= blkno that's
-	 * not all-visible according to the visibility map, or nblocks if there's
-	 * no such block.  Also, we set up the skipping_all_visible_blocks flag,
+	 * next_not_all_visible_block is the next block number >= blkno that's not
+	 * all-visible according to the visibility map, or nblocks if there's no
+	 * such block.	Also, we set up the skipping_all_visible_blocks flag,
 	 * which is needed because we need hysteresis in the decision: once we've
 	 * started skipping blocks, we may as well skip everything up to the next
 	 * not-all-visible block.
@@ -755,8 +725,7 @@ lazy_scan_heap(Relation onerel, LVRelStats *vacrelstats,
 		if (nfrozen > 0)
 		{
 			MarkBufferDirty(buf);
-			/* no XLOG for temp tables, though */
-			if (!onerel->rd_istemp)
+			if (RelationNeedsWAL(onerel))
 			{
 				XLogRecPtr	recptr;
 
@@ -776,7 +745,6 @@ lazy_scan_heap(Relation onerel, LVRelStats *vacrelstats,
 		{
 			/* Remove tuples from heap */
 			lazy_vacuum_page(onerel, blkno, buf, 0, vacrelstats);
-			has_dead_tuples = false;
 
 			/*
 			 * Forget the now-vacuumed tuples, and press on, but be careful
@@ -795,15 +763,16 @@ lazy_scan_heap(Relation onerel, LVRelStats *vacrelstats,
 			PageSetAllVisible(page);
 			SetBufferCommitInfoNeedsSave(buf);
 		}
+
 		/*
 		 * It's possible for the value returned by GetOldestXmin() to move
 		 * backwards, so it's not wrong for us to see tuples that appear to
 		 * not be visible to everyone yet, while PD_ALL_VISIBLE is already
 		 * set. The real safe xmin value never moves backwards, but
 		 * GetOldestXmin() is conservative and sometimes returns a value
-		 * that's unnecessarily small, so if we see that contradiction it
-		 * just means that the tuples that we think are not visible to
-		 * everyone yet actually are, and the PD_ALL_VISIBLE flag is correct.
+		 * that's unnecessarily small, so if we see that contradiction it just
+		 * means that the tuples that we think are not visible to everyone yet
+		 * actually are, and the PD_ALL_VISIBLE flag is correct.
 		 *
 		 * There should never be dead tuples on a page with PD_ALL_VISIBLE
 		 * set, however.
@@ -844,8 +813,8 @@ lazy_scan_heap(Relation onerel, LVRelStats *vacrelstats,
 		/*
 		 * If we remembered any tuples for deletion, then the page will be
 		 * visited again by lazy_vacuum_heap, which will compute and record
-		 * its post-compaction free space.  If not, then we're done with this
-		 * page, so remember its free space as-is.  (This path will always be
+		 * its post-compaction free space.	If not, then we're done with this
+		 * page, so remember its free space as-is.	(This path will always be
 		 * taken if there are no indexes.)
 		 */
 		if (vacrelstats->num_dead_tuples == prev_dead_count)
@@ -859,18 +828,9 @@ lazy_scan_heap(Relation onerel, LVRelStats *vacrelstats,
 	/* now we can compute the new value for pg_class.reltuples */
 	vacrelstats->new_rel_tuples = vac_estimate_reltuples(onerel, false,
 														 nblocks,
-														 vacrelstats->scanned_pages,
+												  vacrelstats->scanned_pages,
 														 num_tuples);
 
-	/*
-	 * Release any remaining pin on visibility map page.
-	 */
-	if (BufferIsValid(vmbuffer))
-	{
-		ReleaseBuffer(vmbuffer);
-		vmbuffer = InvalidBuffer;
-	}
-
 	/* If any tuples need to be deleted, perform final vacuum cycle */
 	/* XXX put a threshold on min number of tuples here? */
 	if (vacrelstats->num_dead_tuples > 0)
@@ -888,6 +848,13 @@ lazy_scan_heap(Relation onerel, LVRelStats *vacrelstats,
 		vacrelstats->num_index_scans++;
 	}
 
+	/* Release the pin on the visibility map page */
+	if (BufferIsValid(vmbuffer))
+	{
+		ReleaseBuffer(vmbuffer);
+		vmbuffer = InvalidBuffer;
+	}
+
 	/* Do post-vacuum cleanup and statistics update for each index */
 	for (i = 0; i < nindexes; i++)
 		lazy_cleanup_index(Irel[i], indstats[i], vacrelstats);
@@ -1009,7 +976,7 @@ lazy_vacuum_page(Relation onerel, BlockNumber blkno, Buffer buffer,
 	MarkBufferDirty(buffer);
 
 	/* XLOG stuff */
-	if (!onerel->rd_istemp)
+	if (RelationNeedsWAL(onerel))
 	{
 		XLogRecPtr	recptr;
 
@@ -1092,9 +1059,7 @@ lazy_cleanup_index(Relation indrel,
 	if (!stats->estimated_count)
 		vac_update_relstats(indrel,
 							stats->num_pages, stats->num_index_tuples,
-							false,
-							InvalidTransactionId,
-							false);
+							false, InvalidTransactionId);
 
 	ereport(elevel,
 			(errmsg("index \"%s\" now contains %.0f row versions in %u pages",
@@ -1120,117 +1085,80 @@ lazy_truncate_heap(Relation onerel, LVRelStats *vacrelstats)
 	BlockNumber old_rel_pages = vacrelstats->rel_pages;
 	BlockNumber new_rel_pages;
 	PGRUsage	ru0;
-	int			lock_retry;
 
 	pg_rusage_init(&ru0);
 
 	/*
-	 * Loop until no more truncating can be done.
+	 * We need full exclusive lock on the relation in order to do truncation.
+	 * If we can't get it, give up rather than waiting --- we don't want to
+	 * block other backends, and we don't want to deadlock (which is quite
+	 * possible considering we already hold a lower-grade lock).
 	 */
-	do
+	if (!ConditionalLockRelation(onerel, AccessExclusiveLock))
+		return;
+
+	/*
+	 * Now that we have exclusive lock, look to see if the rel has grown
+	 * whilst we were vacuuming with non-exclusive lock.  If so, give up; the
+	 * newly added pages presumably contain non-deletable tuples.
+	 */
+	new_rel_pages = RelationGetNumberOfBlocks(onerel);
+	if (new_rel_pages != old_rel_pages)
 	{
 		/*
-		 * We need full exclusive lock on the relation in order to do
-		 * truncation. If we can't get it, give up rather than waiting --- we
-		 * don't want to block other backends, and we don't want to deadlock
-		 * (which is quite possible considering we already hold a lower-grade
-		 * lock).
-		 */
-		vacrelstats->lock_waiter_detected = false;
-		lock_retry = 0;
-		while (true)
-		{
-			if (ConditionalLockRelation(onerel, AccessExclusiveLock))
-				break;
-
-			/*
-			 * Check for interrupts while trying to (re-)acquire the exclusive
-			 * lock.
-			 */
-			CHECK_FOR_INTERRUPTS();
-
-			if (++lock_retry > (VACUUM_TRUNCATE_LOCK_TIMEOUT /
-								VACUUM_TRUNCATE_LOCK_WAIT_INTERVAL))
-			{
-				/*
-				 * We failed to establish the lock in the specified number of
-				 * retries. This means we give up truncating.
-				 */
-				vacrelstats->lock_waiter_detected = true;
-				ereport(elevel,
-						(errmsg("\"%s\": stopping truncate due to conflicting lock request",
-								RelationGetRelationName(onerel))));
-				return;
-			}
-
-			pg_usleep(VACUUM_TRUNCATE_LOCK_WAIT_INTERVAL);
-		}
-
-		/*
-		 * Now that we have exclusive lock, look to see if the rel has grown
-		 * whilst we were vacuuming with non-exclusive lock.  If so, give up;
-		 * the newly added pages presumably contain non-deletable tuples.
-		 */
-		new_rel_pages = RelationGetNumberOfBlocks(onerel);
-		if (new_rel_pages != old_rel_pages)
-		{
-			/*
-			 * Note: we intentionally don't update vacrelstats->rel_pages with
-			 * the new rel size here.  If we did, it would amount to assuming
-			 * that the new pages are empty, which is unlikely. Leaving the
-			 * numbers alone amounts to assuming that the new pages have the
-			 * same tuple density as existing ones, which is less unlikely.
-			 */
-			UnlockRelation(onerel, AccessExclusiveLock);
-			return;
-		}
-
-		/*
-		 * Scan backwards from the end to verify that the end pages actually
-		 * contain no tuples.  This is *necessary*, not optional, because
-		 * other backends could have added tuples to these pages whilst we
-		 * were vacuuming.
-		 */
-		new_rel_pages = count_nondeletable_pages(onerel, vacrelstats);
-
-		if (new_rel_pages >= old_rel_pages)
-		{
-			/* can't do anything after all */
-			UnlockRelation(onerel, AccessExclusiveLock);
-			return;
-		}
-
-		/*
-		 * Okay to truncate.
-		 */
-		RelationTruncate(onerel, new_rel_pages);
-
-		/*
-		 * We can release the exclusive lock as soon as we have truncated.
-		 * Other backends can't safely access the relation until they have
-		 * processed the smgr invalidation that smgrtruncate sent out ... but
-		 * that should happen as part of standard invalidation processing once
-		 * they acquire lock on the relation.
+		 * Note: we intentionally don't update vacrelstats->rel_pages with the
+		 * new rel size here.  If we did, it would amount to assuming that the
+		 * new pages are empty, which is unlikely.	Leaving the numbers alone
+		 * amounts to assuming that the new pages have the same tuple density
+		 * as existing ones, which is less unlikely.
 		 */
 		UnlockRelation(onerel, AccessExclusiveLock);
+		return;
+	}
 
-		/*
-		 * Update statistics.  Here, it *is* correct to adjust rel_pages
-		 * without also touching reltuples, since the tuple count wasn't
-		 * changed by the truncation.
-		 */
-		vacrelstats->pages_removed += old_rel_pages - new_rel_pages;
-		vacrelstats->rel_pages = new_rel_pages;
+	/*
+	 * Scan backwards from the end to verify that the end pages actually
+	 * contain no tuples.  This is *necessary*, not optional, because other
+	 * backends could have added tuples to these pages whilst we were
+	 * vacuuming.
+	 */
+	new_rel_pages = count_nondeletable_pages(onerel, vacrelstats);
 
-		ereport(elevel,
-				(errmsg("\"%s\": truncated %u to %u pages",
-						RelationGetRelationName(onerel),
-						old_rel_pages, new_rel_pages),
-				 errdetail("%s.",
-						   pg_rusage_show(&ru0))));
-		old_rel_pages = new_rel_pages;
-	} while (new_rel_pages > vacrelstats->nonempty_pages &&
-			 vacrelstats->lock_waiter_detected);
+	if (new_rel_pages >= old_rel_pages)
+	{
+		/* can't do anything after all */
+		UnlockRelation(onerel, AccessExclusiveLock);
+		return;
+	}
+
+	/*
+	 * Okay to truncate.
+	 */
+	RelationTruncate(onerel, new_rel_pages);
+
+	/*
+	 * We can release the exclusive lock as soon as we have truncated.	Other
+	 * backends can't safely access the relation until they have processed the
+	 * smgr invalidation that smgrtruncate sent out ... but that should happen
+	 * as part of standard invalidation processing once they acquire lock on
+	 * the relation.
+	 */
+	UnlockRelation(onerel, AccessExclusiveLock);
+
+	/*
+	 * Update statistics.  Here, it *is* correct to adjust rel_pages without
+	 * also touching reltuples, since the tuple count wasn't changed by the
+	 * truncation.
+	 */
+	vacrelstats->rel_pages = new_rel_pages;
+	vacrelstats->pages_removed = old_rel_pages - new_rel_pages;
+
+	ereport(elevel,
+			(errmsg("\"%s\": truncated %u to %u pages",
+					RelationGetRelationName(onerel),
+					old_rel_pages, new_rel_pages),
+			 errdetail("%s.",
+					   pg_rusage_show(&ru0))));
 }
 
 /*
@@ -1242,10 +1170,6 @@ static BlockNumber
 count_nondeletable_pages(Relation onerel, LVRelStats *vacrelstats)
 {
 	BlockNumber blkno;
-	instr_time	starttime;
-
-	/* Initialize the starttime if we check for conflicting lock requests */
-	INSTR_TIME_SET_CURRENT(starttime);
 
 	/* Strange coding of loop control is needed because blkno is unsigned */
 	blkno = vacrelstats->rel_pages;
@@ -1257,38 +1181,6 @@ count_nondeletable_pages(Relation onerel, LVRelStats *vacrelstats)
 					maxoff;
 		bool		hastup;
 
-		/*
-		 * Check if another process requests a lock on our relation. We are
-		 * holding an AccessExclusiveLock here, so they will be waiting. We
-		 * only do this once per VACUUM_TRUNCATE_LOCK_CHECK_INTERVAL, and we
-		 * only check if that interval has elapsed once every 32 blocks to
-		 * keep the number of system calls and actual shared lock table
-		 * lookups to a minimum.
-		 */
-		if ((blkno % 32) == 0)
-		{
-			instr_time	currenttime;
-			instr_time	elapsed;
-
-			INSTR_TIME_SET_CURRENT(currenttime);
-			elapsed = currenttime;
-			INSTR_TIME_SUBTRACT(elapsed, starttime);
-			if ((INSTR_TIME_GET_MICROSEC(elapsed) / 1000)
-				>= VACUUM_TRUNCATE_LOCK_CHECK_INTERVAL)
-			{
-				if (LockHasWaitersRelation(onerel, AccessExclusiveLock))
-				{
-					ereport(elevel,
-							(errmsg("\"%s\": suspending truncate due to conflicting lock request",
-									RelationGetRelationName(onerel))));
-
-					vacrelstats->lock_waiter_detected = true;
-					return blkno;
-				}
-				starttime = currenttime;
-			}
-		}
-
 		/*
 		 * We don't insert a vacuum delay point here, because we have an
 		 * exclusive lock on the table which we want to hold for as short a
