@@ -7,11 +7,11 @@
  *	AccessExclusiveLocks and starting snapshots for Hot Standby mode.
  *	Plus conflict recovery processing.
  *
- * Portions Copyright (c) 1996-2010, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2011, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *	  $PostgreSQL: pgsql/src/backend/storage/ipc/standby.c,v 1.27.2.2 2010/08/19 22:55:10 tgl Exp $
+ *	  src/backend/storage/ipc/standby.c
  *
  *-------------------------------------------------------------------------
  */
@@ -21,7 +21,6 @@
 #include "access/xact.h"
 #include "access/xlog.h"
 #include "miscadmin.h"
-#include "pgstat.h"
 #include "storage/bufmgr.h"
 #include "storage/lmgr.h"
 #include "storage/proc.h"
@@ -82,7 +81,7 @@ InitRecoveryTransactionEnvironment(void)
 	 * hold AccessShareLocks so never block while we write or lock new rows.
 	 */
 	vxid.backendId = MyBackendId;
-	vxid.localTransactionId = MyProc->lxid = GetNextLocalTransactionId();
+	vxid.localTransactionId = GetNextLocalTransactionId();
 	VirtualXactLockTableInsert(vxid);
 
 	standbyState = STANDBY_INITIALIZED;
@@ -98,18 +97,11 @@ InitRecoveryTransactionEnvironment(void)
 void
 ShutdownRecoveryTransactionEnvironment(void)
 {
-	VirtualTransactionId vxid;
-
 	/* Mark all tracked in-progress transactions as finished. */
 	ExpireAllKnownAssignedTransactionIds();
 
 	/* Release all locks the tracked transactions were holding */
 	StandbyReleaseAllLocks();
-
-	/* Cleanup our VirtualTransaction */
-	vxid.backendId = MyBackendId;
-	vxid.localTransactionId = MyProc->lxid;
-	VirtualXactLockTableDelete(vxid);
 }
 
 
@@ -132,7 +124,7 @@ GetStandbyLimitTime(void)
 
 	/*
 	 * The cutoff time is the last WAL data receipt time plus the appropriate
-	 * delay variable.  Delay of -1 means wait forever.
+	 * delay variable.	Delay of -1 means wait forever.
 	 */
 	GetXLogReceiptTime(&rtime, &fromStream);
 	if (fromStream)
@@ -193,16 +185,18 @@ static void
 ResolveRecoveryConflictWithVirtualXIDs(VirtualTransactionId *waitlist,
 									   ProcSignalReason reason)
 {
+	TimestampTz waitStart;
+	char	   *new_status;
+
+	/* Fast exit, to avoid a kernel call if there's no work to be done. */
+	if (!VirtualTransactionIdIsValid(*waitlist))
+		return;
+
+	waitStart = GetCurrentTimestamp();
+	new_status = NULL;			/* we haven't changed the ps display */
+
 	while (VirtualTransactionIdIsValid(*waitlist))
 	{
-		TimestampTz waitStart;
-		char	   *new_status;
-
-		pgstat_report_waiting(true);
-
-		waitStart = GetCurrentTimestamp();
-		new_status = NULL;		/* we haven't changed the ps display */
-
 		/* reset standbyWait_us for each xact we wait for */
 		standbyWait_us = STANDBY_INITIAL_WAIT_US;
 
@@ -248,17 +242,16 @@ ResolveRecoveryConflictWithVirtualXIDs(VirtualTransactionId *waitlist,
 			}
 		}
 
-		/* Reset ps display if we changed it */
-		if (new_status)
-		{
-			set_ps_display(new_status, false);
-			pfree(new_status);
-		}
-		pgstat_report_waiting(false);
-
 		/* The virtual transaction is gone now, wait for the next one */
 		waitlist++;
 	}
+
+	/* Reset ps display if we changed it */
+	if (new_status)
+	{
+		set_ps_display(new_status, false);
+		pfree(new_status);
+	}
 }
 
 void
@@ -341,7 +334,6 @@ static void
 ResolveRecoveryConflictWithLock(Oid dbOid, Oid relOid)
 {
 	VirtualTransactionId *backends;
-	bool		report_memory_error = false;
 	bool		lock_acquired = false;
 	int			num_attempts = 0;
 	LOCKTAG		locktag;
@@ -361,11 +353,8 @@ ResolveRecoveryConflictWithLock(Oid dbOid, Oid relOid)
 		if (++num_attempts < 3)
 			backends = GetLockConflicts(&locktag, AccessExclusiveLock);
 		else
-		{
 			backends = GetConflictingVirtualXIDs(InvalidTransactionId,
 												 InvalidOid);
-			report_memory_error = true;
-		}
 
 		ResolveRecoveryConflictWithVirtualXIDs(backends,
 											 PROCSIG_RECOVERY_CONFLICT_LOCK);
@@ -520,10 +509,6 @@ CheckRecoveryConflictDeadlock(void)
  * RelationLockList, so we can keep track of the various entries made by
  * the Startup process's virtual xid in the shared lock table.
  *
- * We record the lock against the top-level xid, rather than individual
- * subtransaction xids. This means AccessExclusiveLocks held by aborted
- * subtransactions are not released as early as possible on standbys.
- *
  * List elements use type xl_rel_lock, since the WAL record type exactly
  * matches the information that we need to keep track of.
  *
@@ -539,9 +524,7 @@ StandbyAcquireAccessExclusiveLock(TransactionId xid, Oid dbOid, Oid relOid)
 	LOCKTAG		locktag;
 
 	/* Already processed? */
-	if (!TransactionIdIsValid(xid) ||
-		TransactionIdDidCommit(xid) ||
-		TransactionIdDidAbort(xid))
+	if (TransactionIdDidCommit(xid) || TransactionIdDidAbort(xid))
 		return;
 
 	elog(trace_recovery(DEBUG4),
@@ -623,18 +606,23 @@ StandbyReleaseLockTree(TransactionId xid, int nsubxids, TransactionId *subxids)
 }
 
 /*
- * Called at end of recovery and when we see a shutdown checkpoint.
+ * StandbyReleaseLocksMany
+ *		Release standby locks held by XIDs < removeXid
+ *
+ * If keepPreparedXacts is true, keep prepared transactions even if
+ * they're older than removeXid
  */
-void
-StandbyReleaseAllLocks(void)
+static void
+StandbyReleaseLocksMany(TransactionId removeXid, bool keepPreparedXacts)
 {
 	ListCell   *cell,
 			   *prev,
 			   *next;
 	LOCKTAG		locktag;
 
-	elog(trace_recovery(DEBUG2), "release all standby locks");
-
+	/*
+	 * Release all matching locks.
+	 */
 	prev = NULL;
 	for (cell = list_head(RecoveryLockList); cell; cell = next)
 	{
@@ -642,67 +630,10 @@ StandbyReleaseAllLocks(void)
 
 		next = lnext(cell);
 
-		elog(trace_recovery(DEBUG4),
-			 "releasing recovery lock: xid %u db %u rel %u",
-			 lock->xid, lock->dbOid, lock->relOid);
-		SET_LOCKTAG_RELATION(locktag, lock->dbOid, lock->relOid);
-		if (!LockRelease(&locktag, AccessExclusiveLock, true))
-			elog(LOG,
-				 "RecoveryLockList contains entry for lock no longer recorded by lock manager: xid %u database %u relation %u",
-				 lock->xid, lock->dbOid, lock->relOid);
-		RecoveryLockList = list_delete_cell(RecoveryLockList, cell, prev);
-		pfree(lock);
-	}
-}
-
-/*
- * StandbyReleaseOldLocks
- *		Release standby locks held by top-level XIDs that aren't running,
- *		as long as they're not prepared transactions.
- */
-void
-StandbyReleaseOldLocks(int nxids, TransactionId *xids)
-{
-	ListCell   *cell,
-			   *prev,
-			   *next;
-	LOCKTAG		locktag;
-
-	prev = NULL;
-	for (cell = list_head(RecoveryLockList); cell; cell = next)
-	{
-		xl_standby_lock *lock = (xl_standby_lock *) lfirst(cell);
-		bool	remove = false;
-
-		next = lnext(cell);
-
-		Assert(TransactionIdIsValid(lock->xid));
-
-		if (StandbyTransactionIdIsPrepared(lock->xid))
-			remove = false;
-		else
-		{
-			int		i;
-			bool	found = false;
-
-			for (i = 0; i < nxids; i++)
-			{
-				if (lock->xid == xids[i])
-				{
-					found = true;
-					break;
-				}
-			}
-
-			/*
-			 * If its not a running transaction, remove it.
-			 */
-			if (!found)
-				remove = true;
-		}
-
-		if (remove)
+		if (!TransactionIdIsValid(removeXid) || TransactionIdPrecedes(lock->xid, removeXid))
 		{
+			if (keepPreparedXacts && StandbyTransactionIdIsPrepared(lock->xid))
+				continue;
 			elog(trace_recovery(DEBUG4),
 				 "releasing recovery lock: xid %u db %u rel %u",
 				 lock->xid, lock->dbOid, lock->relOid);
@@ -719,6 +650,27 @@ StandbyReleaseOldLocks(int nxids, TransactionId *xids)
 	}
 }
 
+/*
+ * Called at end of recovery and when we see a shutdown checkpoint.
+ */
+void
+StandbyReleaseAllLocks(void)
+{
+	elog(trace_recovery(DEBUG2), "release all standby locks");
+	StandbyReleaseLocksMany(InvalidTransactionId, false);
+}
+
+/*
+ * StandbyReleaseOldLocks
+ *		Release standby locks held by XIDs < removeXid, as long
+ *		as they're not prepared transactions.
+ */
+void
+StandbyReleaseOldLocks(TransactionId removeXid)
+{
+	StandbyReleaseLocksMany(removeXid, true);
+}
+
 /*
  * --------------------------------------------------------------------
  *		Recovery handling for Rmgr RM_STANDBY_ID
@@ -860,16 +812,9 @@ standby_desc(StringInfo buf, uint8 xl_info, char *rec)
  * Later, when we apply the running xact data we must be careful to ignore
  * transactions already committed, since those commits raced ahead when
  * making WAL entries.
- *
- * The loose timing also means that locks may be recorded that have a
- * zero xid, since xids are removed from procs before locks are removed.
- * So we must prune the lock list down to ensure we hold locks only for
- * currently running xids, performed by StandbyReleaseOldLocks().
- * Zero xids should no longer be possible, but we may be replaying WAL
- * from a time when they were possible.
  */
 void
-LogStandbySnapshot(void)
+LogStandbySnapshot(TransactionId *nextXid)
 {
 	RunningTransactions running;
 	xl_standby_lock *locks;
@@ -879,11 +824,16 @@ LogStandbySnapshot(void)
 
 	/*
 	 * Get details of any AccessExclusiveLocks being held at the moment.
+	 *
+	 * XXX GetRunningTransactionLocks() currently holds a lock on all
+	 * partitions though it is possible to further optimise the locking. By
+	 * reference counting locks and storing the value on the ProcArray entry
+	 * for each backend we can easily tell if any locks need recording without
+	 * trying to acquire the partition locks and scanning the lock table.
 	 */
 	locks = GetRunningTransactionLocks(&nlocks);
 	if (nlocks > 0)
 		LogAccessExclusiveLocks(nlocks, locks);
-	pfree(locks);
 
 	/*
 	 * Log details of all in-progress transactions. This should be the last
@@ -893,6 +843,8 @@ LogStandbySnapshot(void)
 	LogCurrentRunningXacts(running);
 	/* GetRunningTransactionData() acquired XidGenLock, we must release it */
 	LWLockRelease(XidGenLock);
+
+	*nextXid = running->nextXid;
 }
 
 /*
@@ -1007,14 +959,14 @@ void
 LogAccessExclusiveLockPrepare(void)
 {
 	/*
-	 * Ensure that a TransactionId has been assigned to this transaction,
-	 * for two reasons, both related to lock release on the standby.
-	 * First, we must assign an xid so that RecordTransactionCommit() and
+	 * Ensure that a TransactionId has been assigned to this transaction, for
+	 * two reasons, both related to lock release on the standby. First, we
+	 * must assign an xid so that RecordTransactionCommit() and
 	 * RecordTransactionAbort() do not optimise away the transaction
-	 * completion record which recovery relies upon to release locks. It's
-	 * a hack, but for a corner case not worth adding code for into the
-	 * main commit path. Second, must must assign an xid before the lock
-	 * is recorded in shared memory, otherwise a concurrently executing
+	 * completion record which recovery relies upon to release locks. It's a
+	 * hack, but for a corner case not worth adding code for into the main
+	 * commit path. Second, must must assign an xid before the lock is
+	 * recorded in shared memory, otherwise a concurrently executing
 	 * GetRunningTransactionLocks() might see a lock associated with an
 	 * InvalidTransactionId which we later assert cannot happen.
 	 */
